﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Rendering;
using Microsoft.EntityFrameworkCore;
using FormWebCore.Models.DB;
using FormWebCore.Models;
using FormWebCore.Areas.Maticni.Models;
using FormWebCore.MyCode;
using Syncfusion.EJ2.Base;
using static FormWebCore.Models.MyModels;
using System.Collections;
using System.Web;
using System.Net;
using Newtonsoft.Json.Linq;

namespace FormWebCore.Areas.Maticni.Controllers
{
    [Area("Maticni")]
    public class FirmasController : Controller
    {
        private readonly FormWebCoreContext _context;

        public FirmasController(FormWebCoreContext context)
        {
            _context = context;
        }

        // GET: Maticni/Firmas
        [HttpGet]
        public IActionResult Index()
        {
            var dsFirme = from f in _context.Firma
                          join m in _context.Mjesto on f.MjestoId equals m.MjestoId into m1
                          from m2 in m1.DefaultIfEmpty()
                          join u in _context.Ulica on f.UlicaId equals u.UlicaId into u1
                          from u2 in u1.DefaultIfEmpty()
                          join v in _context.FirmaVrsta on f.VrstaId equals v.VrstaId into v1
                          from v2 in v1.DefaultIfEmpty()
                          orderby f.Naziv + " " + v2.Naziv
                          select new
                          {
                              f.FirmaId,
                              Naziv = v2.Naziv == null ? f.Naziv : f.Naziv + " " + v2.Naziv,
                              f.Oib,
                              UlicaKbr = u2.Naziv + " " + f.Kbr,
                              Mjesto = m2.Posta + " " + m2.Naziv,
                              f.Telefon,
                              f.Email
                          };

            ViewBag.dsFirme = dsFirme.ToList();
            return PartialView("_Index");
        }

        [HttpGet]
        public IActionResult CreateEdit(int? id)
        {
            var dsMjesto = from m in _context.Mjesto
                           orderby m.Naziv
                           select new
                           {
                               m.MjestoId,
                               m.Naziv
                           };
            var dsUlica = from u in _context.Ulica
                           orderby u.Naziv
                           select new
                           {
                               u.UlicaId,
                               u.Naziv
                           };
            var dsVrsta = from v in _context.FirmaVrsta
                          orderby v.Naziv
                          select new
                          {
                              v.VrstaId,
                              v.Naziv
                          };
            var dsBanka = from b in _context.Banka
                          select new
                          {
                              b.BankaId,
                              b.Naziv
                          };
            var dsFirma = from f in _context.Firma
                          select new
                          {
                              f.FirmaId,
                              f.Naziv
                          };
            var dsRacuni = (from r in _context.FirmaRacun
                            where r.FirmaId == id
                            select new
                            {
                                r.RacunId,
                                r.FirmaId,
                                r.BankaId,
                                r.Iban,
                                r.IntBrojFirme,
                                r.Aktivan
                            }).ToList();

            ViewBag.dsMjesto = dsMjesto.ToList();
            ViewBag.dsUlica = dsUlica.ToList();
            ViewBag.dsVrsta = dsVrsta.ToList();
            ViewBag.dsRacuni = dsRacuni.ToArray();
            ViewBag.dsBanka = dsBanka.ToList();
            ViewBag.dsFirma = dsFirma.ToList();

            if (id == null)
            {
                Firma firma = new Firma();
                firma.Aktivna = true;
                firma.SustavPdv = true;

                return PartialView("_CreateEdit", firma);
            }
            else
            {
                Firma firma = _context.Firma.Find(id);

                if (firma == null)
                {
                    return NotFound();
                }

                return PartialView("_CreateEdit", firma);
            }
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public async Task<IActionResult> CreateEdit(Firma firma, string dsFirmaRacuni)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    if (firma.FirmaId > 0)
                    {
                        _context.Update(firma);
                        await _context.SaveChangesAsync();
                    }
                    else {
                        _context.Add(firma);
                        await _context.SaveChangesAsync();
                    }

                    // **************************
                    // SPREMANJE RAČUNA FIRME
                    // **************************

                    int id = 0;
                    JArray arr = JArray.Parse(dsFirmaRacuni);
                    foreach (JObject obj in arr)
                    {
                        if (Convert.ToDecimal(obj["RacunId"].ToString().Replace(".", ",")) < 1)
                        {
                            id -= 1;
							obj["FirmaId"] = firma.FirmaId;
                            obj["RacunId"] = id;
                        }
                    }
                    var json2 = Newtonsoft.Json.JsonConvert.SerializeObject(arr);
                    var deserFirmaRacun = (List<FirmaRacun>)Newtonsoft.Json.JsonConvert.DeserializeObject(json2, typeof(List<FirmaRacun>));

                    string ppr = "";
                    if (deserFirmaRacun.Count == 0)
                    {
                        var tmp = _context.Database.ExecuteSqlCommand("DELETE FROM FirmaRacun WHERE FirmaID = " + firma.FirmaId.ToString());
                    }
                    foreach (var pp in deserFirmaRacun)
                    {
                        if (pp.RacunId >= 1)
                            ppr = ppr + "," + pp.RacunId.ToString();
                    }
                    if (ppr != "")
                    {
                        ppr = ppr.Substring(1);
                        var tmp = _context.Database.ExecuteSqlCommand("DELETE FROM FirmaRacun WHERE FirmaID = " + firma.FirmaId.ToString() + " AND RacunId NOT IN (" + ppr + ")");
                    }

                    foreach (var pp in deserFirmaRacun)
                    {
                        FirmaRacun ppp = _context.FirmaRacun.Find(pp.RacunId);
                        if (ppp == null)
                        {
                            ppp = new FirmaRacun();
                            ppp.FirmaId = firma.FirmaId;
                            ppp.BankaId = (int)pp.BankaId;
                            ppp.Iban = pp.Iban;
                            ppp.IntBrojFirme = pp.IntBrojFirme;
                            ppp.Aktivan = (bool)pp.Aktivan;
                            _context.FirmaRacun.Add(ppp);
                        }
                        else
                        {
                            ppp.BankaId = (int)pp.BankaId;
                            ppp.Iban = pp.Iban;
                            ppp.IntBrojFirme = pp.IntBrojFirme;
                            ppp.Aktivan = (bool)pp.Aktivan;
                            _context.Entry(ppp).State = EntityState.Modified;
                        }
                        _context.SaveChanges();
                    }
                    // **************************
                    // **************************

                    return RedirectToAction(nameof(Index));
                }
            }
            catch (DbUpdateException e)
            {
                ModelState.AddModelError("", "Greška kod spremanja: " + e.InnerException.Message);
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", "Greška kod spremanja: " + ex.Message);
            }

            if (ModelState.ErrorCount == 0)
            {
                return RedirectToAction("Index", "Firmas", new { area = "Maticni" });
            }

            var dsMjesto = from m in _context.Mjesto
                           orderby m.Naziv
                           select new
                           {
                               m.MjestoId,
                               m.Naziv
                           };
            var dsUlica = from u in _context.Ulica
                          orderby u.Naziv
                          select new
                          {
                              u.UlicaId,
                              u.Naziv
                          };
            var dsVrsta = from v in _context.FirmaVrsta
                          orderby v.Naziv
                          select new
                          {
                              v.VrstaId,
                              v.Naziv
                          };

            ViewBag.dsMjesto = dsMjesto.ToList();
            ViewBag.dsUlica = dsUlica.ToList();
            ViewBag.dsVrsta = dsVrsta.ToList();

            return PartialView("_CreateEdit", firma);
        }
		
        private bool FirmaExists(int id)
        {
            return _context.Firma.Any(e => e.FirmaId == id);
        }
    }
}
